require "./selenium_spec/spec_helper.rb"

TAB_NAMES = ['Summary', 'group', 'host', 'package', 'user']
RESOURCE_TABS = TAB_NAMES.reject {|x| x == "Summary"}
FIND_RESOURCE_TIMEOUT = 60
NODES_LIST_TIMEOUT = 20

describe 'the Live Management web UI Manage Resources tab' do

  before :all do
    @driver = get_driver
    @menu = @driver.find_element(:css => "div#main div#content ul.navigation")
    @tabs = @menu.find_elements(:tag_name => "li")
    @panel = @driver.find_element(:css => "div#main div#content div.panel")
  end

  after :all do
    @driver.quit
  end

  describe 'resource types menu' do
      
    it 'should exist' do
      @menu.displayed?.should == true
    end

    it "should have exactly #{TAB_NAMES.length} tabs" do
      @tabs.length.should == TAB_NAMES.length
    end

    TAB_NAMES.each_with_index do |tab_name, i|
      it "should have a #{tab_name} tab" do
        @tabs[i].text.should == tab_name
      end
    end

    it 'should have the Summary tab active by default' do
      active_tab.text.should == "Summary"
    end

    it 'should activate a tab when the tab is clicked' do
      @tabs.each do |tab|
        link = tab.find_element(:css => "a")
        @driver.action.click(link).perform
        tab.should == active_tab
      end
    end

  end

  describe 'Summary panel' do

    before :all do
      @summary_link = @driver.find_element(:css => "div#main div#content ul.navigation li a")
      @driver.action.click(@summary_link).perform
      @summaries = @panel.find_elements(:class => "summary")
    end

    it 'should have summaries of all resource tabs in the same order as the sidebar menu' do
      @summaries.each_with_index do |summary, i|
        resource_name = summary.find_element(:css => "a strong").text
        resource_name.should == TAB_NAMES[i + 1]
      end
      @summaries.length.should == TAB_NAMES.length - 1
    end

    describe 'resource summaries' do
      RESOURCE_TABS.each_with_index do |tab_name, i|
        it "should link to the #{tab_name} resource panel" do
          @driver.action.click(@summary_link).perform
          resource_link = @panel.find_element(:css => (["div.summary"]*(i + 1)).join(" + ") + " a")
          @driver.action.click(resource_link).perform
          active_tab.text.should == tab_name
        end
      end
    end

  end

  describe 'resource type' do

    RESOURCE_TABS.each_with_index do |tab_name, i|

      describe "#{tab_name} panel" do

        it "should still be in the same position in the tab menu" do
          tab = @tabs[i + 1]
          tab.text.should == tab_name
          link = tab.find_element(:css => "a")
          @driver.action.click(link).perform
        end

        it "should display #{tab_name} in the title" do
          resource_type = @panel.find_element(:css => "div.title h2 strong").text
          resource_type.should == tab_name
        end

        it "should have an empty search field and a search button" do
          button_input = @panel.find_element(:css => "div.title div.resource-search form input[type=submit].btn")
          button_input.attribute("value").should == "Search"
          text_input = @panel.find_element(:css => "div.title div.resource-search form input[type=text]")
          text_input.attribute("value").should == ""
        end

        describe 'results table' do

          it "should load within #{FIND_RESOURCE_TIMEOUT} seconds after clicking the Find Resources button" do
            find_resources_button = @panel.find_element(:css => "div.find-resources button")
            wait = Selenium::WebDriver::Wait.new(:timeout => NODES_LIST_TIMEOUT, 
                                                 :message => "Nodes list didn't load within #{NODES_LIST_TIMEOUT} seconds")
            wait.until { find_resources_button.enabled? }
            @driver.action.click(find_resources_button).perform
            wait = Selenium::WebDriver::Wait.new(:timeout => FIND_RESOURCE_TIMEOUT)
            wait.until {
              results_table = @panel.find_element(:css => "table.sortable")
              headers = results_table.find_elements(:css => "thead tr th")
              headers.map(&:text) == ['Name', 'Nodes', 'Variations']
            }.should be_true
          end

          it "should have a number of rows equal to the number of unique resources" do
            unique_resources = Integer(@panel.find_element(:css => "div.similarity-summary ul.counts li strong").text)
            results_table = @panel.find_element(:css => "table.sortable")
            rows = results_table.find_elements(:css => "tbody tr")
            rows.length.should == unique_resources
          end

          it "should have a nodes column that sums to the number of total hosts" do
            total_hosts = Integer(@panel.find_element(:css => "div.similarity-summary ul.counts li + li strong").text)
            results_table = @panel.find_element(:css => "table.sortable")
            node_counts = results_table.find_elements(:css => "tbody tr > td:nth-child(2) a").map {|tag| Integer(tag.text)}
            node_counts.reduce(0){|sum,item| sum + item}.should == total_hosts
          end

          it "should have a similarity between 0% and 100%" do
            similarity = @panel.find_element(:css => "div.similarity-summary div.similarity h4 strong").text
            similarity.end_with?('%').should == true
            similarity = Float(similarity[0..-2])
            similarity.should <= 100
            similarity.should >= 0
          end

          it "should load a randomly selected result within #{FIND_RESOURCE_TIMEOUT} seconds" do
            results_table = @panel.find_element(:css => "table.sortable")
            rows = results_table.find_elements(:css => "tbody tr")
            row = rows.choice
            @@resource_title = row.find_element(:css => "td").text
            @@resource_node_count = Integer(row.find_element(:css => "td + td").text)
            @@resource_variation_count = Integer(row.find_element(:css => "td + td + td").text)
            @driver.action.click(row.find_element(:tag_name => "a")).perform
            wait = Selenium::WebDriver::Wait.new(:timeout => FIND_RESOURCE_TIMEOUT)
            wait.until {
              variations = @panel.find_element(:css => "table.variations")
              variations.displayed?
            }.should == true
          end

        end

        describe "variations table for the randomly selected #{tab_name}" do

          before :all do
            @variations = @panel.find_element(:css => "table.variations")
          end

          it "should display the same resource title as the results table" do
            title =  @panel.find_element(:css => "div + p + div + div h4 span[data-bind=title]").text
            title.should == @@resource_title
          end

          it "should display the same node count as the results table" do
            Integer(@panel.find_element(:css => 'span[data-bind="resource.nodeCount | default \'no\'"]').text).should == @@resource_node_count
          end

          it "should display the same variation count as the results table" do
            Integer(@panel.find_element(:css => 'span[data-bind="resource.variations | get \'length\'"]').text).should == @@resource_variation_count
          end

          it "should have one column for property names and one column for each variation" do
            @variations.find_elements(:css => "thead > tr:nth-child(1) th").length.should == @@resource_variation_count + 1
          end

          it "should have a search field populated with the #{tab_name} title" do
            text_input = @panel.find_element(:css => "div.title div.resource-search form input[type=text]")
            text_input.attribute("value").should == @@resource_title
          end

          it "should have a 'Return to #{tab_name} list' link" do
            link = @panel.find_element(:partial_link_text => "Return to")
            link.text.end_with?("Return to #{tab_name} list").should == true
            @driver.action.click(link).perform
            @panel.find_element(:css => "table.sortable").should_not be_nil
          end

        end

        describe "search bar" do

          before :all do
            @text_input = @panel.find_element(:css => "div.title div.resource-search form input[type=text]")
            results_table = @panel.find_element(:css => "table.sortable")
            first_resource = results_table.find_element(:css => "tbody tr")
            @@resource_title = first_resource.find_element(:css => "td").text
            @@resource_node_count = Integer(first_resource.find_element(:css => "td + td").text)
            @@resource_variation_count = Integer(first_resource.find_element(:css => "td + td + td").text)
          end

          it "should have an empty text field" do
            @text_input.attribute("value").should == ""
          end

          it "should display the first #{tab_name} within #{FIND_RESOURCE_TIMEOUT} seconds after entering the #{tab_name} title and submitting" do
            @text_input.send_keys(@@resource_title)
            @text_input.submit()
            wait = Selenium::WebDriver::Wait.new(:timeout => FIND_RESOURCE_TIMEOUT)
            wait.until {
              variations = @panel.find_element(:css => "table.variations")
              variations.displayed?
            }.should == true
          end

          it "should have a text field populated with the #{tab_name} title" do
            text_input = @panel.find_element(:css => "div.title div.resource-search form input[type=text]")
            text_input.attribute("value").should == @@resource_title
          end

        end
      end
    end
  end

  def active_tab
    active_tabs = @menu.find_elements(:class => "active")
    raise "There is more than one active tab." if active_tabs.length != 1
    active_tabs.first
  end

end
